package java.util;

import java.util.spi.LocaleServiceProvider;
import java.security.AccessController;
import java.io.FileReader;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.security.PrivilegedAction;
import java.util.concurrent.ConcurrentHashMap;
import sun.util.logging.PlatformLogger;
import java.text.SimpleDateFormat;
import java.util.regex.Matcher;
import java.text.ParseException;
import java.util.regex.Pattern;
import java.io.IOException;
import java.io.DataInputStream;
import sun.util.locale.provider.LocaleServiceProviderPool;
import java.util.spi.CurrencyNameProvider;
import java.util.concurrent.ConcurrentMap;
import java.io.Serializable;

public final class Currency implements Serializable
{
    private static final long serialVersionUID = -158308464356906721L;
    private final String currencyCode;
    private final transient int defaultFractionDigits;
    private final transient int numericCode;
    private static ConcurrentMap<String, Currency> instances;
    private static HashSet<Currency> available;
    static int formatVersion;
    static int dataVersion;
    static int[] mainTable;
    static long[] scCutOverTimes;
    static String[] scOldCurrencies;
    static String[] scNewCurrencies;
    static int[] scOldCurrenciesDFD;
    static int[] scNewCurrenciesDFD;
    static int[] scOldCurrenciesNumericCode;
    static int[] scNewCurrenciesNumericCode;
    static String otherCurrencies;
    static int[] otherCurrenciesDFD;
    static int[] otherCurrenciesNumericCode;
    private static final int MAGIC_NUMBER = 1131770436;
    private static final int A_TO_Z = 26;
    private static final int INVALID_COUNTRY_ENTRY = 127;
    private static final int COUNTRY_WITHOUT_CURRENCY_ENTRY = 512;
    private static final int SIMPLE_CASE_COUNTRY_MASK = 0;
    private static final int SIMPLE_CASE_COUNTRY_FINAL_CHAR_MASK = 31;
    private static final int SIMPLE_CASE_COUNTRY_DEFAULT_DIGITS_MASK = 480;
    private static final int SIMPLE_CASE_COUNTRY_DEFAULT_DIGITS_SHIFT = 5;
    private static final int SIMPLE_CASE_COUNTRY_MAX_DEFAULT_DIGITS = 9;
    private static final int SPECIAL_CASE_COUNTRY_MASK = 512;
    private static final int SPECIAL_CASE_COUNTRY_INDEX_MASK = 31;
    private static final int SPECIAL_CASE_COUNTRY_INDEX_DELTA = 1;
    private static final int COUNTRY_TYPE_MASK = 512;
    private static final int NUMERIC_CODE_MASK = 1047552;
    private static final int NUMERIC_CODE_SHIFT = 10;
    private static final int VALID_FORMAT_VERSION = 2;
    private static final int SYMBOL = 0;
    private static final int DISPLAYNAME = 1;
    
    private Currency(String currencyCode, int defaultFractionDigits, int numericCode) {
        this.currencyCode = currencyCode;
        this.defaultFractionDigits = defaultFractionDigits;
        this.numericCode = numericCode;
    }
    
    public static Currency getInstance(String s) {
        return getInstance(s, Integer.MIN_VALUE, 0);
    }
    
    private static Currency getInstance(String str, int n, int n2) {
        Currency currency = (Currency)Currency.instances.get(str);
        if (currency != null) {
            return currency;
        }
        if (n == Integer.MIN_VALUE) {
            if (str.length() != 3) {
                throw new IllegalArgumentException();
            }
            int mainTableEntry = getMainTableEntry(str.charAt(0), str.charAt(1));
            if ((mainTableEntry & 0x200) == 0x0 && mainTableEntry != 127 && str.charAt(2) - 65 == (mainTableEntry & 0x1F)) {
                n = (mainTableEntry & 0x1E0) >> 5;
                n2 = (mainTableEntry & 0xFFC00) >> 10;
            }
            else {
                if (str.charAt(2) == '-') {
                    throw new IllegalArgumentException();
                }
                int index = Currency.otherCurrencies.indexOf(str);
                if (index == -1) {
                    throw new IllegalArgumentException();
                }
                n = Currency.otherCurrenciesDFD[index / 4];
                n2 = Currency.otherCurrenciesNumericCode[index / 4];
            }
        }
        Currency o = new Currency(str, n, n2);
        Currency currency2 = (Currency)Currency.instances.putIfAbsent(str, o);
        return (currency2 != null) ? currency2 : o;
    }
    
    public static Currency getInstance(Locale locale) {
        String country = locale.getCountry();
        if (country == null) {
            throw new NullPointerException();
        }
        if (country.length() != 2) {
            throw new IllegalArgumentException();
        }
        int mainTableEntry = getMainTableEntry(country.charAt(0), country.charAt(1));
        if ((mainTableEntry & 0x200) == 0x0 && mainTableEntry != 127) {
            int c = (char)((mainTableEntry & 0x1F) + 65);
            int n2 = (mainTableEntry & 0x1E0) >> 5;
            int n3 = (mainTableEntry & 0xFFC00) >> 10;
            StringBuilder sb = new StringBuilder(country);
            sb.append((char)c);
            return getInstance(sb.toString(), n2, n3);
        }
        if (mainTableEntry == 127) {
            throw new IllegalArgumentException();
        }
        if (mainTableEntry == 512) {
            return null;
        }
        int n = (mainTableEntry & 0x1F) - 1;
        if (Currency.scCutOverTimes[n] == Long.MAX_VALUE || System.currentTimeMillis() < Currency.scCutOverTimes[n]) {
            return getInstance(Currency.scOldCurrencies[n], Currency.scOldCurrenciesDFD[n], Currency.scOldCurrenciesNumericCode[n]);
        }
        return getInstance(Currency.scNewCurrencies[n], Currency.scNewCurrenciesDFD[n], Currency.scNewCurrenciesNumericCode[n]);
    }
    
    public static Set<Currency> getAvailableCurrencies() {
        synchronized (Currency.class) {
            if (Currency.available == null) {
                Currency.available = new HashSet(256);
                for (int i = 'A'; i <= 'Z'; i = (char)(i + 1)) {
                    for (int j = 'A'; j <= 'Z'; j = (char)(j + 1)) {
                        int mainTableEntry = getMainTableEntry((char)i, (char)j);
                        if ((mainTableEntry & 0x200) == 0x0 && mainTableEntry != 127) {
                            int c = (char)((mainTableEntry & 0x1F) + 65);
                            int n = (mainTableEntry & 0x1E0) >> 5;
                            int n2 = (mainTableEntry & 0xFFC00) >> 10;
                            StringBuilder o = new StringBuilder();
                            o.append((char)i);
                            o.append((char)j);
                            o.append((char)c);
                            Currency.available.add(getInstance(o.toString(), n, n2));
                        }
                    }
                }
                StringTokenizer o2 = new StringTokenizer(Currency.otherCurrencies, "-");
                while (o2.hasMoreElements()) {
                    Currency.available.add(getInstance((String)o2.nextElement()));
                }
            }
        }
        return (Set)Currency.available.clone();
    }
    
    public String getCurrencyCode() {
        return this.currencyCode;
    }
    
    public String getSymbol() {
        return this.getSymbol(Locale.getDefault(Locale.Category.DISPLAY));
    }
    
    public String getSymbol(Locale locale) {
        String s = (String)LocaleServiceProviderPool.getPool(CurrencyNameProvider.class).getLocalizedObject(Currency.CurrencyNameGetter.INSTANCE, locale, this.currencyCode, new Object[] { Integer.valueOf(0) });
        if (s != null) {
            return s;
        }
        return this.currencyCode;
    }
    
    public int getDefaultFractionDigits() {
        return this.defaultFractionDigits;
    }
    
    public int getNumericCode() {
        return this.numericCode;
    }
    
    public String getDisplayName() {
        return this.getDisplayName(Locale.getDefault(Locale.Category.DISPLAY));
    }
    
    public String getDisplayName(Locale locale) {
        String s = (String)LocaleServiceProviderPool.getPool(CurrencyNameProvider.class).getLocalizedObject(Currency.CurrencyNameGetter.INSTANCE, locale, this.currencyCode, new Object[] { Integer.valueOf(1) });
        if (s != null) {
            return s;
        }
        return this.currencyCode;
    }
    
    @Override
    public String toString() {
        return this.currencyCode;
    }
    
    private Object readResolve() {
        return getInstance(this.currencyCode);
    }
    
    private static int getMainTableEntry(char c, char c2) {
        if (c < 'A' || c > 'Z' || c2 < 'A' || c2 > 'Z') {
            throw new IllegalArgumentException();
        }
        return Currency.mainTable[(c - 65) * 26 + (c2 - 65)];
    }
    
    private static void setMainTableEntry(char c, char c2, int n) {
        if (c < 'A' || c > 'Z' || c2 < 'A' || c2 > 'Z') {
            throw new IllegalArgumentException();
        }
        Currency.mainTable[(c - 65) * 26 + (c2 - 65)] = n;
        return;
    }
    
    private static int[] readIntArray(DataInputStream dataInputStream, int n) throws IOException {
        int[] array = new int[n];
        for (int i = 0; i < n; ++i) {
            array[i] = dataInputStream.readInt();
        }
        return array;
    }
    
    private static long[] readLongArray(DataInputStream dataInputStream, int n) throws IOException {
        long[] array = new long[n];
        for (int i = 0; i < n; ++i) {
            array[i] = dataInputStream.readLong();
        }
        return array;
    }
    
    private static String[] readStringArray(DataInputStream dataInputStream, int n) throws IOException {
        String[] array = new String[n];
        for (int i = 0; i < n; ++i) {
            array[i] = dataInputStream.readUTF();
        }
        return array;
    }
    
    private static void replaceCurrencyData(Pattern pattern, String str, String str2) {
        if (str.length() != 2) {
            info("currency.properties entry for " + str + " is ignored because of the invalid country code.", null);
            return;
        }
        Matcher matcher = pattern.matcher(str2);
        if (!matcher.find() || (matcher.group(4) == null && countOccurrences(str2, ',') >= 3)) {
            info("currency.properties entry for " + str + " ignored because the value format is not recognized.", null);
            return;
        }
        try {
            if (matcher.group(4) != null && !isPastCutoverDate(matcher.group(4))) {
                info("currency.properties entry for " + str + " ignored since cutover date has not passed :" + str2, null);
                return;
            }
        }
        catch (ParseException ex) {
            info("currency.properties entry for " + str + " ignored since exception encountered :" + ex.getMessage(), null);
            return;
        }
        String group = matcher.group(1);
        int n = Integer.parseInt(matcher.group(2)) << 10;
        int int1 = Integer.parseInt(matcher.group(3));
        if (int1 > 9) {
            info("currency.properties entry for " + str + " ignored since the fraction is more than " + 9 + ":" + str2, null);
            return;
        }
        int n2;
        for (n2 = 0; n2 < Currency.scOldCurrencies.length && !Currency.scOldCurrencies[n2].equals(group); ++n2) {}
        int n3;
        if (n2 == Currency.scOldCurrencies.length) {
            n3 = (n | (int1 << 5 | group.charAt(2) - 65));
        }
        else {
            n3 = (n | (0x200 | n2 + 1));
        }
        setMainTableEntry(str.charAt(0), str.charAt(1), n3);
        return;
    }
    
    private static boolean isPastCutoverDate(String s) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.ROOT);
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
        simpleDateFormat.setLenient(false);
        return System.currentTimeMillis() > simpleDateFormat.parse(s.trim()).getTime();
    }
    
    private static int countOccurrences(String s, char c) {
        int n = 0;
        char[] charArray = s.toCharArray();
        for (int length = charArray.length, i = 0; i < length; ++i) {
            if (charArray[i] == c) {
                ++n;
            }
        }
        return n;
    }
    
    private static void info(String s, Throwable t) {
        PlatformLogger logger = PlatformLogger.getLogger("java.util.Currency");
        if (logger.isLoggable(PlatformLogger.Level.INFO)) {
            if (t != null) {
                logger.info(s, t);
            }
            else {
                logger.info(s);
            }
        }
    }
    
    static int[] access$000(DataInputStream dataInputStream, int n) throws IOException {
        return readIntArray(dataInputStream, n);
    }
    
    static long[] access$100(DataInputStream dataInputStream, int n) throws IOException {
        return readLongArray(dataInputStream, n);
    }
    
    static String[] access$200(DataInputStream dataInputStream, int n) throws IOException {
        return readStringArray(dataInputStream, n);
    }
    
    static void access$300(Pattern pattern, String s, String s2) {
        replaceCurrencyData(pattern, s, s2);
    }
    
    static void access$400(String s, Throwable t) {
        info(s, t);
    }
    
    static {
        Currency.instances = new ConcurrentHashMap(7);
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                String property = System.getProperty("java.home");
                try (final DataInputStream dataInputStream = new DataInputStream(new BufferedInputStream(new FileInputStream(property + File.separator + "lib" + File.separator + "currency.data")))) {
                    if (dataInputStream.readInt() != 1131770436) {
                        throw new InternalError("Currency data is possibly corrupted");
                    }
                    Currency.formatVersion = dataInputStream.readInt();
                    if (Currency.formatVersion != 2) {
                        throw new InternalError("Currency data format is incorrect");
                    }
                    Currency.dataVersion = dataInputStream.readInt();
                    Currency.mainTable = Currency.access$000(dataInputStream, 676);
                    int int1 = dataInputStream.readInt();
                    Currency.scCutOverTimes = Currency.access$100(dataInputStream, int1);
                    Currency.scOldCurrencies = Currency.access$200(dataInputStream, int1);
                    Currency.scNewCurrencies = Currency.access$200(dataInputStream, int1);
                    Currency.scOldCurrenciesDFD = Currency.access$000(dataInputStream, int1);
                    Currency.scNewCurrenciesDFD = Currency.access$000(dataInputStream, int1);
                    Currency.scOldCurrenciesNumericCode = Currency.access$000(dataInputStream, int1);
                    Currency.scNewCurrenciesNumericCode = Currency.access$000(dataInputStream, int1);
                    int int2 = dataInputStream.readInt();
                    Currency.otherCurrencies = dataInputStream.readUTF();
                    Currency.otherCurrenciesDFD = Currency.access$000(dataInputStream, int2);
                    Currency.otherCurrenciesNumericCode = Currency.access$000(dataInputStream, int2);
                }
                catch (IOException cause) {
                    throw new InternalError(cause);
                }
                String pathname = System.getProperty("java.util.currency.data");
                if (pathname == null) {
                    pathname = property + File.separator + "lib" + File.separator + "currency.properties";
                }
                try {
                    File file = new File(pathname);
                    if (file.exists()) {
                        Properties o = new Properties();
                        try (final Object o2 = new FileReader(file)) {
                            o.load(o2);
                        }
                        Set<String> stringPropertyNames = o.stringPropertyNames();
                        Pattern compile = Pattern.compile("([A-Z]{3})\\s*,\\s*(\\d{3})\\s*,\\s*(\\d+)\\s*,?\\s*(\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2})?");
                        for (String s : stringPropertyNames) {
                            Currency.access$300(compile, s.toUpperCase(Locale.ROOT), o.getProperty(s).toUpperCase(Locale.ROOT));
                        }
                    }
                }
                catch (IOException ex) {
                    Currency.access$400("currency.properties is ignored because of an IOException", ex);
                }
                return null;
            }
        });
    }
    
    private static class CurrencyNameGetter implements LocaleServiceProviderPool.LocalizedObjectGetter<CurrencyNameProvider, String>
    {
        private static final Currency.CurrencyNameGetter INSTANCE;
        
        @Override
        public String getObject(CurrencyNameProvider currencyNameProvider, Locale locale, String currencyCode, Object... array) {
            assert array.length == 1;
            switch ((int)array[0]) {
                case 0: {
                    return currencyNameProvider.getSymbol(currencyCode, locale);
                }
                case 1: {
                    return currencyNameProvider.getDisplayName(currencyCode, locale);
                }
                default: {
                    assert false;
                    return null;
                }
            }
        }
        
        static {
            INSTANCE = new Currency.CurrencyNameGetter();
        }
    }
}
